`include "defines.v"

`include "SAI_input.v"
`include "DF1_FIR_CORE.v"
`include "SAI_output.v"

	
module DF1
(
// global clock/reset inputs
input clk, 				// global master-clock input
input clrn, 			// global nReset input (active low)
 
// ------------------------  Config ports -------------------------------------------------------------------------
// clock config
input mck_mode,			// input clock multiplicity: 0 - 1024Fs, 1 - 768Fs
input mck_div,			// otput mck divider: 0 - clk/1, 1 - clk/2
input oscsel,			// osc clock domen select: 0 - osc1ena, 1 - osc2ena

// FIR filter config
input lp_mode,			// 1 - enable low power mode
input[1:0] att,			// attenuation input signal: 0 - 0db, 1 - 1db, 2 - 2db, 3 - 3db
input[1:0] ovs_max, 	// max oversampling 0..3 - x4..x32

// input config
input[1:0] format, 		// 0 - I2S, 1 - LJ, 2 - RJ24, 3 - RJ16
input[1:0] frame,		// input frame lenth: 0 (or 3) - 64bit, 1 - 48bit, 2 - 32bit
input mute,				// soft mute input signal (active high)

// output data format config
input ob_ntwc, 			// 1 - offset binary, 0 - 2's complement
input bck_inv, 			// 1 - force bck inversion
input[2:0] d_width, 	// data lenth 0..4 - 16bit..24 bit (2bit step), 5 - 32bit, 6..7 - bck_continue mode (24bit - right justified)
input[2:0] d_speed, 	// data rate 0..5 - clk/1, clk/2, clk/3, clk/4, clk/6, clk/8
input[1:0] spi_dsize,	// SPI data field lenth 0 - = d_width, 1 - 16bit, 2 - 20bit, 3 - 24bit
input[2:0] spi_mode,	// 0 - diff, 1 - diff serial, 2 - conv, 3 - conv serial, 4 - channels serial
input[2:0] head_mode,	// 0 - no header, 1 - DAC8812 head, 2 - AD5791 head, 3 - AD5063 head, 4 - DAC11001 head, 5 - LTC1592 head, 6 - LTC2752 head, 7 - reserved
input outZ,				// 1 - force spi output both channals (dat, bck and wck) to Z-state.
input outInv,			// 1 - invert serial outputs both channels: dat, wck and lutch.

// deglitcher config
input dg_inv,			// deglitcher signal invertion
input dg_pp,			// deglitcher pipeline mode
input dg_off,			// logic 1 - turn deglitching sirciut to always sample state (off state)
input[1:0] dgh_mode, 	// deglitcher rise front mode
input[1:0] dgl_mode,	// deglitcher fall front mode if spi_dsize == 0 then (0 - rise wck, 1 - fall wck, 2 - pcm1704 mode, 3 - pcm1702 mode), else (0 - rise wck, 1 - fall latch)
// ----------------------------------------------------------------------------------------------------------------

// led indicator outputs (active low)
output[2:0] ovs_ind,
output clipping,

// SAI input data signals
input sai_dat, sai_sck, sai_lrc,

// output SPI (or PCM) data signals
output dat1_left, dat1_right, bck1, wck1, ltch1,
output dat2_left, dat2_right, bck2, wck2, ltch2,
output dg,

// clock control outputs
output osc1ena, osc2ena,	// oscillator controls
output mcko					// master-clock output for feedback sync
);


// control registers
reg[16:0] rclip;

// SAI_input outputs
wire sai_in_rdy;
wire signed[`BUS_WIDTH-1:0] sai_d_left, sai_d_right;

// fir_lp_stereo_27bit outputs
wire signed[23:0] fir_out_left, fir_out_right;
wire[2:0] wfs_in;
wire wclip, wbypass;

// SAI_Output outputs
wire sai_out_load;

// output assigments
assign osc1ena = (~oscsel);
assign osc2ena = oscsel;
assign ovs_ind = (wbypass) ? 0 : (~wfs_in);
assign clipping = rclip[16];

wire sclip = ~rclip[16] & sai_in_rdy;

// digital clipping nidication
always@( posedge clk or negedge wclip ) begin
	rclip <= (!wclip) ? 0 : (rclip + sclip);
end


SAI_input __sai_input(
	// global clock/reset inputs
	.clk( clk ), .clrn( clrn ),
	// control inputs
	.mute( mute ),
	.format( format ), .frame( frame ), .att( att ),
	// serial data input
	.sai_dat( sai_dat ), .sai_sck( sai_sck ), .sai_lrc( sai_lrc ),

	// parallel data output
	.q_left( sai_d_left ), .q_right( sai_d_right ),
	.ready( sai_in_rdy )
);

DF1_FIR_CORE __fir(
	// global clock/reset inputs
	.clk( clk ), .clrn( clrn ), .mck_mode( mck_mode ), .mck_div( mck_div ),
	// setting inputs
	.sai_lrc( sai_lrc ), .lp_mode( lp_mode ), .dithen( 1'b1 ), .shpen( 1'b1 ),
	.ovs_max( ovs_max ), .att( att ),
	.d_width( d_width ),
	// control inputs
	.in_ready( sai_in_rdy ), .out_load( sai_out_load ),
	// input data
	.d_left( sai_d_left ), .d_right( sai_d_right ),

	// output data
	.q_left( fir_out_left ), .q_right( fir_out_right ),
	.fs_in( wfs_in ),
	.mcko( mcko ), .clipn( wclip ), .bypass( wbypass )
);

SAI_output __sai_output(
	// input global signals
	.clk( clk ), .clrn( clrn ), .mck_mode( mck_mode ),
	// input control signals
	.ob_ntwc( ob_ntwc ), .bck_inv( bck_inv ), .dg_inv( dg_inv ), .dg_pp( dg_pp ), .dg_off( dg_off ), .outZ( outZ ), .outInv( outInv ),
	.ovs_max( ovs_max ), .spi_dsize( spi_dsize ), .dgh_mode( dgh_mode ), .dgl_mode( dgl_mode ),
	.d_width( d_width ), .d_speed( d_speed ), .spi_mode( spi_mode ), .head_mode( head_mode ),
	// input data
	.d_left( fir_out_left ), .d_right( fir_out_right ),

	// output control
	.load( sai_out_load ),
	// output data
	.dat1_left( dat1_left ), .dat1_right( dat1_right ), .bck1( bck1 ), .wck1( wck1 ), .ltch1( ltch1 ),
	.dat2_left( dat2_left ), .dat2_right( dat2_right ), .bck2( bck2 ), .wck2( wck2 ), .ltch2( ltch2 ),
	.dg( dg )
);












endmodule







